//
//  StatusXMLParser.m
//  TwitterViewer
//
//  Created by Ichi Kanaya on 10/11/01.
//  This code is based on sample code at http://www.atmarkit.co.jp/fwcr/design/tool/iphonesdk04/01.html
//

#import "StatusXMLParser.h"


@implementation StatusXMLParser

@synthesize currentXpath;
@synthesize statuses;
@synthesize currentStatus;
@synthesize textNodeCharacters;
@synthesize attributes;

- (void)parserDidStartDocument: (NSXMLParser *)parser {
	self.currentXpath = [[[NSMutableString alloc] init] autorelease];
	self.statuses = [[[NSMutableArray alloc] init] autorelease];
	self.attributes = nil;
}

- (void)parser: (NSXMLParser *)parser didStartElement: (NSString *)elementName namespaceURI: (NSString *)namespaceURI qualifiedName: (NSString *)qName attributes: (NSDictionary *)attributeDict {
	[self.currentXpath appendString: elementName];
	[self.currentXpath appendString: @"/"];
	self.textNodeCharacters = [[[NSMutableString alloc] init] autorelease];
	if ([self.currentXpath isEqualToString: @"feed/entry/"]) {
		self.currentStatus = [[[NSMutableDictionary alloc] init] autorelease];
	}
	else if ([self.currentXpath isEqualToString: @"feed/entry/link/"]) {
		self.attributes = [attributeDict retain];
	}
}

- (void)parser: (NSXMLParser *)parser didEndElement: (NSString *)elementName namespaceURI: (NSString *)namespaceURI qualifiedName: (NSString *)qName {	
	NSString *textData = [self.textNodeCharacters stringByTrimmingCharactersInSet: [NSCharacterSet whitespaceAndNewlineCharacterSet]];
	if ([self.currentXpath isEqualToString: @"feed/entry/"]) {
		[self.statuses addObject: self.currentStatus];
		self.currentStatus = nil;
	} 
	else if ([self.currentXpath isEqualToString: @"feed/entry/title/"]) {
		[self.currentStatus setValue: textData forKey: @"text"];
		NSLog(@"text: %@", textData);
	} 
	else if ([self.currentXpath isEqualToString: @"feed/entry/author/name/"]) {
		[self.currentStatus setValue: textData forKey: @"name"];
		NSLog(@"name: %@", textData);
	}
	else if ([self.currentXpath isEqualToString: @"feed/entry/link/"]) {
		if (self.attributes) {
			NSString *linkText = [self.attributes objectForKey: @"href"];
			if ([linkText hasPrefix: @"http://twitter.com/"]) {
				NSURL *link = [NSURL URLWithString: linkText];
				NSLog(@"link: %@", [self.attributes objectForKey: @"href"]);
				[self.currentStatus setValue: link forKey: @"link"];
			}
			[self.attributes release];
		}
	}
	int delLength = [elementName length] + 1;
	int delIndex = [self.currentXpath length] - delLength;
	[self.currentXpath deleteCharactersInRange: NSMakeRange(delIndex, delLength)];
}

- (void)parser: (NSXMLParser *)parser foundCharacters: (NSString *)string {
	[self.textNodeCharacters appendString: string];
}

- (NSArray *)parseStatuses: (NSData *)xmlData {
	NSXMLParser *parser = [[[NSXMLParser alloc] initWithData: xmlData] autorelease];
	[parser setDelegate: self];
	[parser parse];
	return self.statuses;
}

- (void) dealloc {
	[currentXpath release];
	[statuses release];
	[currentStatus release];
	[textNodeCharacters release];
	[super dealloc];
}

@end